/*
	This file is part of lgui.

	lgui is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lgui is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lgui.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "liststore.h"

int lgui_list_store_new(lua_State* L)
{
	int size = lua_gettop(L), i;
	GType* types = g_malloc(sizeof(GType) * size);

	/* Fill the types array */
	for(i = 1; i <= size; i++)
	{
		types[i - 1] = g_type_from_name(luaL_checkstring(L, i));
	}

	GtkListStore* ptr = gtk_list_store_newv(size, types);
	object_new(L, ptr, TRUE);
	g_free(types);

	return 1;
}

int lgui_list_store_append(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gtk_list_store_append(GTK_LIST_STORE(obj->pointer), (GtkTreeIter*)stc->pointer);

	return 0;
}

int lgui_list_store_prepend(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gtk_list_store_prepend(GTK_LIST_STORE(obj->pointer), (GtkTreeIter*)stc->pointer);

	return 0;
}

int lgui_list_store_insert(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TNUMBER);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gtk_list_store_insert(GTK_LIST_STORE(obj->pointer), (GtkTreeIter*)stc->pointer,
	lua_tointeger(L, 3));

	return 0;
}

int lgui_list_store_remove(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	Object* obj = lua_touserdata(L, 1);
	Object* stc = lua_touserdata(L, 2);

	gboolean res = gtk_list_store_remove(GTK_LIST_STORE(obj->pointer), (GtkTreeIter*)stc->pointer);
	lua_pushboolean(L, res);

	return 1;
}

int lgui_list_store_swap(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);
	luaL_checktype(L, 3, LUA_TUSERDATA);

	Object* obj1 = lua_touserdata(L, 1);
	Object* obj2 = lua_touserdata(L, 2);
	Object* obj3 = lua_touserdata(L, 3);

	gtk_list_store_swap(GTK_LIST_STORE(obj1->pointer), (GtkTreeIter*)obj2->pointer,
		(GtkTreeIter*)obj3->pointer);

	return 0;
}

int lgui_list_store_clear(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);

	Object* list = lua_touserdata(L, 1);
	gtk_list_store_clear(GTK_LIST_STORE(list->pointer));

	return 0;
}

int lgui_list_store_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int size = lua_gettop(L), i, curr;
	GtkListStore* list = GTK_LIST_STORE(((Object*)lua_touserdata(L, 1))->pointer);
	GtkTreeIter*  iter = (GtkTreeIter*)((Object*)lua_touserdata(L, 2))->pointer;
	lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValueSet");

	/* {model, iter, key, val, ...} */
	int nvals = (size - 2) / 2;
	 
	/* Alocate the temp GValue array  */
	int vsize = sizeof(GValue) * nvals;
	GValue* vals = g_malloc(vsize);
	GValue* val;
	memset(vals, 0, vsize);
	
	/* Allocate the columns array */
	int* cols = g_malloc(sizeof(int) * nvals);
 	
	/* Get the fields by field */
	for(i = 3; i <= size; i += 2)
	{
		luaL_checknumber(L, i);
		luaL_checkany(L, i + 1);

		int column = lua_tointeger(L, i);
		GType type = gtk_tree_model_get_column_type(GTK_TREE_MODEL(list), column);

		#ifdef IDEBUG
		g_debug("Setting listStore column %i, of type %s!", column, g_type_name(type));
		#endif
		
		/* Current pair */
		curr = (i - 3) / 2;
		cols[curr] = column;
		val = &vals[curr];
		
		g_value_init(val, type);
		lua_pushvalue(L, size + 1); /* "__lobjValueSet" */
		lua_pushlightuserdata(L, val);
		lua_pushvalue(L, i + 1);
		lua_call(L, 2, 0);
	}
	
	/* Ok, now we have the array with the values. Set them with one hit! */
	gtk_list_store_set_valuesv(list, iter, cols, vals, nvals);
	
	/* Now unset them all */
	for(i = 0; i < nvals; ++i)
		g_value_unset(&vals[i]);
		
	/* Free the dinamic alocated arrays */
	g_free(vals);
	g_free(cols);

	return 0;
}

int lgui_list_store_seto(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TUSERDATA);

	int size = lua_gettop(L), i;
	GtkListStore* list = GTK_LIST_STORE(((Object*)lua_touserdata(L, 1))->pointer);
	GtkTreeIter*  iter = (GtkTreeIter*)((Object*)lua_touserdata(L, 2))->pointer;
	lua_getfield(L, LUA_GLOBALSINDEX, "__lobjValueSet");

	/* {model, iter, val, val, ...} */
	int nvals = (size - 2);
	 
	/* Alocate the temp GValue array  */
	int vsize = sizeof(GValue) * nvals;
	GValue* vals = g_malloc(vsize);
	GValue* val;
	memset(vals, 0, vsize);
	
	/* Allocate the columns array */
	int* cols = g_malloc(sizeof(int) * nvals);
 	
	/* Get the fields by field */
	for(i = 3; i <= size; ++i)
	{
		int column = i - 3;
		GType type = gtk_tree_model_get_column_type(GTK_TREE_MODEL(list), column);

		#ifdef IDEBUG
		g_debug("Setting listStore column %i, of type %s!", column, g_type_name(type));
		#endif
		
		/* Current pair */
		cols[column] = column;
		val = &vals[column];
		
		g_value_init(val, type);
		lua_pushvalue(L, size + 1); /* "__lobjValueSet" */
		lua_pushlightuserdata(L, val);
		lua_pushvalue(L, i);
		lua_call(L, 2, 0);
	}
	
	/* Ok, now we have the array with the values. Set them with one hit! */
	gtk_list_store_set_valuesv(list, iter, cols, vals, nvals);
	
	/* Now unset them all */
	for(i = 0; i < nvals; ++i)
		g_value_unset(&vals[i]);
		
	/* Free the dinamic alocated arrays */
	g_free(vals);
	g_free(cols);

	return 0;
}
